//----------------------------------------------------------------------------------
// AVR ADC in equivalent time sampling mode demo. 
//----------------------------------------------------------------------------------
// AVR generates waveform on one PWM channel and samples the waveform with its own 
// ADC in equivalent time sampling mode. Sampled waveform is stored into buffer
// which can be read via UART command 'W'. See project project WWW for more details.
//
// V1.00, 2013.08.01, Stanislav Maslan, s.maslan@seznam.cz
// http://elektronika.kvalitne.cz/ATMEL/necoteorie/AVR_ETS_bandwidth/AVR_ETS_bandwidth.html
//----------------------------------------------------------------------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/atomic.h>

// --- Serial interface, uses RTS handshake ---
#include "serial.h"


// --- USART ---
#define BAUD 1250000

// --- OCR setup ---
// clock divisor id (1 for /1, 2 for /8, ...)
#define CCKDIV 1
// timer MAX value
#define CCKOCR 830u

// --- ADC clk/(2^ADCPS) ---
#define ADCPS 7

// --- ADC setup ---
// total samples per period
#define WSMPL 831u
// subsets per period
#define WSMPLF 2u


//----------------------------------------------------------------------------------
// ADC sample ready
//----------------------------------------------------------------------------------
uint16_t adc_buf[WSMPL];
uint16_t *adc_buf_p;
ISR(ADC_vect)
{
	// --- ADC sample ---

	// use local copy of the pointer (less AVRGCC rubbish)
	uint16_t *p = adc_buf_p;

	// store new sample
	*p++ = ADC;
	// update address
	p += (WSMPLF-1);
	// reset pointer to buffer start?
	if(p >= &adc_buf[WSMPL])
		p -= WSMPL;

	adc_buf_p = p;
}


//----------------------------------------------------------------------------------
// MAIN
//----------------------------------------------------------------------------------
int main(void)
{
	// disable WDT if enabled (sometimes it fucks up after reset ...)
	cli();
	wdt_reset();
	MCUSR = 0;
	WDTCSR = (1<<WDCE)|(1<<WDE);
	WDTCSR = 0;
	
	// XTAL/2
	/*CLKPR = (1<<CLKPCE);
	CLKPR = (1<<CLKPS0);*/

	// PWM output OC1B
	DDRD = (1<<PD4);

	// Timer 1: PWM waveform generator on OC1B with period OCR1A
	TCCR1A = (3<<COM1B0)|(3<<WGM10);
	TCCR1B = (3<<WGM12)|(CCKDIV<<CS10);
	OCR1A = CCKOCR;
	OCR1B = CCKOCR/2;  // 50% duty-cycle

	// ADC: free running (13 ADC clock cycles/sample)
	DIDR0 = (1<<ADC0D);
	ADMUX = (2<<REFS0)|(0<<MUX0);
	ADCSRB = (0<<ADTS0);
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADATE)|(1<<ADIE)|(ADCPS<<ADPS0);
	adc_buf_p = &adc_buf[0];

	// USART setup (RX, TX, 1.0 STOP, no parity, double speed)
	UCSR0A = (1<<U2X0);
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	UCSR0C = (3<<UCSZ00);
	UBRR0 = F_CPU/(8*BAUD)-1;

	// enable int.
	sei();


	// --- CMD processing loop ---
	while(1)
	{
		uint8_t cmd = rx_byte();
		
		switch(cmd)
		{
			case 'W':; // --- return ADC waveform ---
			
				// return samples count N (word)
				tx_word(WSMPL);
				
				// return sample data (N words)
				uint16_t *buf = adc_buf;
				for(uint16_t i=0;i<WSMPL;i++)
					tx_word(buf[i]);

				break;
	

			default:; // --- invalid command ---

				// send non-ACK
				tx_byte('E');

				break;
		}
		
	}
}
